#ifndef __SW_SPI_H
#define __SW_SPI_H

/* PIC18 Software SPI library header
 *
 * To use the software spi routines, the user must define
 * the port and tris register for each of the CS, DIN, DOUT,
 * and SCK pins.  The SPI mode must also be defined (MODE0,
 * MODE1, MODE2, MODE3).
 *
 * Define the port and pin for each of the software SPI pins
 *  - Chip select pin CS must have a port and tris definition.
 *  - Data in pin DIN must have a port and tris definition.
 *  - Data out pin DOUT must have a port and tris definition.
 *  - Clock pin SCK must have a port and tris definition.
 */
#define SW_CE_PIN         LATCbits.LATC3      // Chip Select
#define TRIS_SW_CE_PIN    TRISCbits.TRISC3
#define SW_DIN_PIN        PORTCbits.RC1     // Data in
#define TRIS_SW_DIN_PIN   TRISCbits.TRISC1
#define SW_DOUT_PIN       LATCbits.LATC2    // Data out
#define TRIS_SW_DOUT_PIN  TRISCbits.TRISC2
#define SW_SCK_PIN        LATCbits.LATC0     // Clock
#define TRIS_SW_SCK_PIN   TRISCbits.TRISC0
 
#define MODE0

void LClearCSSWSPI(void);
void LSetCSSWSPI(void);
void LClearCESWSPI(void);
void LSetCESWSPI(void);
void LOpenSWSPI (void);
char LWriteSWSPI( char output);

#endif