/*******************************************************************************
 FileName:      main.c
 Dependencies:  See INCLUDES section
 Processor:		PIC18 or PIC24 or PIC32 USB Microcontrollers
 Hardware:		The code is natively intended to be used on the following
                hardware platforms: PICDEM?FS USB Demo Board, 
				Low Pin Count USB development Kit,
 				Explorer 16 + PIC24 USB PIM, or Explorer 16 + PIC32 USB PIM
 				The firmware may be modified for use on other USB platforms 
 				by editing the HardwareProfile.h file.
 Complier:  	Microchip C18 (for PIC18), C30 (for PIC24) or C32 (for PIC32)
 Company:		Microchip Technology, Inc.

 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the Company? for its PIC?Microcontroller is intended and
 supplied to you, the Company customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN RS IS?CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************************
 File Description:

 Change History:
  Rev   Date         Description
  1.0   04/27/2009   Initial release
  2.0   08/06/2009   COM3101 v1.0
  2.0T  04/14/2011   COM3101T v1.0 for APP013 HobbyKit (Adam Syu)
*******************************************************************************/

/** INCLUDES ******************************************************************/

#include "Compiler.h"
#include "HardwareProfile.h"
#include "usb_config.h"
#include "USB\usb.h"
#include "USB\usb_function_cdc.h"

/** CONFIGURATION *************************************************************/
#if defined(PICDEM_FS_USB)                  // Configuration bits for PICDEM
                                            // FS USB Demo Board (PIC18F4550)
        #pragma config PLLDIV   = 5         // (20 MHz crystal on PICDEM FS USB
                                            //  board)
        #pragma config CPUDIV   = OSC1_PLL2   
        #pragma config USBDIV   = 2         // Clock source from 96MHz PLL/2
        #pragma config FOSC     = HSPLL_HS
        #pragma config FCMEN    = OFF
        #pragma config IESO     = OFF
        #pragma config PWRT     = OFF
        #pragma config BOR      = ON
        #pragma config BORV     = 3
        #pragma config VREGEN   = ON        // USB Voltage Regulator
        #pragma config WDT      = OFF
        #pragma config WDTPS    = 32768
        #pragma config MCLRE    = ON
        #pragma config LPT1OSC  = OFF
        #pragma config PBADEN   = OFF
        #pragma config STVREN   = ON
        #pragma config LVP      = OFF
        #pragma config XINST    = OFF       // Extended Instruction Set
        #pragma config CP0      = OFF
        #pragma config CP1      = OFF
        #pragma config CPB      = OFF
        #pragma config WRT0     = OFF
        #pragma config WRT1     = OFF
        #pragma config WRTB     = OFF       // Boot Block Write Protection
        #pragma config WRTC     = OFF
        #pragma config EBTR0    = OFF
        #pragma config EBTR1    = OFF
        #pragma config EBTRB    = OFF

#elif defined(LOW_PIN_COUNT_USB_DEVELOPMENT_KIT)
        //14K50
        #pragma config CPUDIV = NOCLKDIV
        #pragma config USBDIV = OFF
        #pragma config FOSC   = HS
        #pragma config PLLEN  = ON
        #pragma config FCMEN  = OFF
        #pragma config IESO   = OFF
        #pragma config PWRTEN = OFF
        #pragma config BOREN  = OFF
        #pragma config BORV   = 30
        #pragma config WDTEN  = OFF
        #pragma config WDTPS  = 32768
        #pragma config MCLRE  = OFF
        #pragma config HFOFST = OFF
        #pragma config STVREN = ON
        #pragma config LVP    = OFF
        #pragma config XINST  = OFF
        #pragma config BBSIZ  = OFF
        #pragma config CP0    = OFF
        #pragma config CP1    = OFF
        #pragma config CPB    = OFF
        #pragma config WRT0   = OFF
        #pragma config WRT1   = OFF
        #pragma config WRTB   = OFF
        #pragma config WRTC   = OFF
        #pragma config EBTR0  = OFF
        #pragma config EBTR1  = OFF
        #pragma config EBTRB  = OFF       

#elif defined(EXPLORER_16)
    #ifdef __PIC24FJ256GB110__              // Defined by MPLAB when using 
                                            // PIC24FJ256GB110 device
        _CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF & ICS_PGx2) 
        _CONFIG2( 0xF7FF & IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_OFF & POSCMOD_HS & FNOSC_PRIPLL & PLLDIV_DIV2 & IOL1WAY_ON)

    #elif defined(__32MX460F512L__)
        #pragma config UPLLEN   = ON        // USB PLL Enabled
        #pragma config FPLLMUL  = MUL_15    // PLL Multiplier
        #pragma config UPLLIDIV = DIV_2     // USB PLL Input Divider
        #pragma config FPLLIDIV = DIV_2     // PLL Input Divider
        #pragma config FPLLODIV = DIV_1     // PLL Output Divider
        #pragma config FPBDIV   = DIV_1     // Peripheral Clock divisor
        #pragma config FWDTEN   = OFF       // Watchdog Timer
        #pragma config WDTPS    = PS1       // Watchdog Timer Postscale
        #pragma config FCKSM    = CSDCMD    // Clock Switching & Fail Safe Clock
                                            // Monitor
        #pragma config OSCIOFNC = OFF       // CLKO Enable
        #pragma config POSCMOD  = HS        // Primary Oscillator
        #pragma config IESO     = OFF       // Internal/External Switch-over
        #pragma config FSOSCEN  = OFF       // Secondary Oscillator Enable
                                            // (KLO was off)
        #pragma config FNOSC    = PRIPLL    // Oscillator Selection
        #pragma config CP       = OFF       // Code Protect
        #pragma config BWP      = OFF       // Boot Flash Write Protect
        #pragma config PWP      = OFF       // Program Flash Write Protect
        #pragma config ICESEL   = ICS_PGx2  // ICE/ICD Comm Channel Select
        #pragma config DEBUG    = ON        // Background Debugger Enable

    #else
        #error No hardware board defined, see "HardwareProfile.h" and __FILE__
    #endif

#elif defined(APP013_HobbyKit_18F47J53) || defined(APP013_HobbyKit_18F46J50)
	#pragma config WDTEN = OFF          //WDT disabled (enabled by SWDTEN bit)
	#pragma config PLLDIV = 3           //Divide by 3 (12 MHz oscillator input)
	#pragma config STVREN = ON            //stack overflow/underflow reset enabled
	#pragma config XINST = OFF          //Extended instruction set disabled
	#pragma config CPUDIV = OSC1        //No CPU system clock divide
	#pragma config CP0 = OFF            //Program memory is not code-protected
	#pragma config OSC = HSPLL          //HS oscillator, PLL enabled, HSPLL used by USB
	//#pragma config T1DIG = ON           //Sec Osc clock source may be selected
	//#pragma config LPT1OSC = OFF        //high power Timer1 mode
	#pragma config FCMEN = OFF          //Fail-Safe Clock Monitor disabled
	#pragma config IESO = OFF           //Two-Speed Start-up disabled
	#pragma config WDTPS = 32768        //1:32768
	#pragma config DSWDTOSC = INTOSCREF //DSWDT uses INTOSC/INTRC as clock
	#pragma config RTCOSC = T1OSCREF    //RTCC uses T1OSC/T1CKI as clock
	#pragma config DSBOREN = OFF        //Zero-Power BOR disabled in Deep Sleep
	#pragma config DSWDTEN = OFF        //Disabled
	#pragma config DSWDTPS = 8192       //1:8,192 (8.5 seconds)
	#pragma config IOL1WAY = OFF        //IOLOCK bit can be set and cleared
	#pragma config MSSP7B_EN = MSK7     //7 Bit address masking
	#pragma config WPFP = PAGE_1        //Write Protect Program Flash Page 0
	#pragma config WPEND = PAGE_0       //Start protection at page 0
	#pragma config WPCFG = OFF          //Write/Erase last page protect Disabled
	#pragma config WPDIS = OFF          //WPFP[5:0], WPEND, and WPCFG bits ignored  

#else
    #error No hardware board defined, see "HardwareProfile.h" and __FILE__
#endif

/** VARIABLES *****************************************************************/
#pragma udata
char USB_Out_Buffer[64];
char USB_In_Buffer[64];
BOOL old_sw;

/** PRIVATE PROTOTYPES ********************************************************/
static void InitializeSystem(void);
void ProcessIO(void);
void UserInit(void);
void Lab3a(void);
void Lab3b(void);
BOOL SwitchIsPressed(void);

/** INTERRUPT VECTORS (C18) ***************************************************/
#if defined(__18CXX)
	#pragma interrupt YourHighPriorityISRCode
	void YourHighPriorityISRCode(void)
	{
		//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flag
		//Etc.
        #if defined(USB_INTERRUPT)
	        USBDeviceTasks();
        #endif
	
	}	//This return will be a "retfie fast", since this is in a #pragma interrupt section 
	#pragma interruptlow YourLowPriorityISRCode
	void YourLowPriorityISRCode(void)
	{
		//Check which interrupt flag caused the interrupt.
		//Service the interrupt
		//Clear the interrupt flag
		//Etc.
	
	}	//This return will be a "retfie", since this is in a #pragma interruptlow section 


	#pragma code HIGH_INTERRUPT_VECTOR = 0x08
	void High_ISR (void)
	{
	     _asm goto YourHighPriorityISRCode _endasm
	}
	#pragma code LOW_INTERRUPT_VECTOR = 0x18
	void Low_ISR (void)
	{
	     _asm goto YourLowPriorityISRCode _endasm
	}
#endif	//end of "#if defined(__18CXX)"

/** DECLARATIONS **************************************************************/
#pragma code

/*******************************************************************************
 * Function:        void main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 ******************************************************************************/
#if defined(__18CXX)
void main(void)
#else
int main(void)
#endif
{   
    InitializeSystem();

    while(1)
    {
 		#if defined(USB_INTERRUPT)
			if(USB_BUS_SENSE && (USBGetDeviceState() == DETACHED_STATE))
                USBDeviceAttach();
		#endif

		#if defined(USB_POLLING)

		// Check bus status and service USB interrupts.
        USBDeviceTasks(); // Using polling, must call this function periodically.  
        				  // This function will take care of processing and 
        				  // responding to SETUP transactions (such as during the 
        				  // enumeration process when you first plug in). 
        				  // USB hosts require that USB devices should accept
        				  // and process SETUP packets in a timely fashion.  
        				  // Therefore, when using polling, this function should 
        				  // be called frequently (such as once about every 
        				  // 100 microseconds) at any time that a SETUP packet 
        				  // might reasonably be expected to be sent by the host 
        				  // to your device.  In most cases, the USBDeviceTasks() 
        				  // function does not take very long to execute 
        				  // (~50 instruction cycles) before it returns.

		// Application-specific tasks.
		// Application related code may be added here,
		// or in the ProcessIO() function.
		#endif

       ProcessIO();        
    }//end while
}//end main


/*******************************************************************************
 * Function:        static void InitializeSystem(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        InitializeSystem is a centralize initialization
 *                  routine. All required USB initialization routines
 *                  are called from here.
 *
 *                  User application initialization routine should
 *                  also be called from here.                  
 *
 * Note:            None
 ******************************************************************************/
static void InitializeSystem(void)
{
    #if defined(__18CXX) 
        ADCON1 |= 0x0F;                 // Default all pins to digital
    #elif defined(__C30__)
        AD1PCFGL = 0xFFFF;
    #elif defined(__C32__)
        DDPCONbits.JTAGEN =0;
        AD1PCFG = 0xFFFF;
    #endif

	#if defined(APP013_HobbyKit_18F47J53) || defined(APP013_HobbyKit_18F46J50)
	{
		unsigned int pll_startup_counter = 600;
		OSCTUNEbits.PLLEN = 1;  //Enable the PLL and wait 2+ms until the PLL locks before enabling USB module
		while(pll_startup_counter--);
	}

	#if defined(APP013_HobbyKit_18F47J53) || defined(APP013_HobbyKit_18F46J50)
	{
 		ANCON0 = 0xFF;                  // Default all pins to digital
		ANCON1 = 0xFF;                  // Default all pins to digital
	}

//	The USB specifications require that USB peripheral devices must never source
//	current onto the Vbus pin.  Additionally, USB peripherals should not source
//	current on D+ or D- when the host/hub is not actively powering the Vbus line.
//	When designing a self powered (as opposed to bus powered) USB peripheral
//	device, the firmware should make sure not to turn on the USB module and D+
//	or D- pull up resistor unless Vbus is actively powered.  Therefore, the
//	firmware needs some means to detect when Vbus is being powered by the host.
//	A 5V tolerant I/O pin can be connected to Vbus (through a resistor), and
// 	can be used to detect when Vbus is high (host actively powering), or low
//	(host is shut down or otherwise not supplying power).  The USB firmware
// 	can then periodically poll this I/O pin to know when it is okay to turn on
//	the USB module/D+/D- pull up resistor.  When designing a purely bus powered
//	peripheral device, it is not possible to source current on D+ or D- when the
//	host is not actively providing power on Vbus. Therefore, implementing this
//	bus sense feature is optional.  This firmware can be made to use this bus
//	sense feature by making sure "USE_USB_BUS_SENSE_IO" has been defined in the
//	HardwareProfile.h file.    
    #if defined(USE_USB_BUS_SENSE_IO)
    tris_usb_bus_sense = INPUT_PIN; // See HardwareProfile.h
    #endif
    
//	If the host PC sends a GetStatus (device) request, the firmware must respond
//	and let the host know if the USB peripheral device is currently bus powered
//	or self powered.  See chapter 9 in the official USB specifications for details
//	regarding this request.  If the peripheral device is capable of being both
//	self and bus powered, it should not return a hard coded value for this request.
//	Instead, firmware should check if it is currently self or bus powered, and
//	respond accordingly.  If the hardware has been configured like demonstrated
//	on the PICDEM FS USB Demo Board, an I/O pin can be polled to determine the
//	currently selected power source.  On the PICDEM FS USB Demo Board, "RA2" 
//	is used for	this purpose.  If using this feature, make sure 
//	"USE_SELF_POWER_SENSE_IO" has been defined in HardwareProfile.h, and that an 
//	appropriate I/O pin has been mapped to it in HardwareProfile.h.
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;	// See HardwareProfile.h
    #endif
    
    UserInit();

    USBDeviceInit();	//usb_device.c.  Initializes USB module SFRs and firmware
    					//variables to known states.
}//end InitializeSystem



/*******************************************************************************
 * Function:        void UserInit(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This routine should take care of all of the demo code
 *                  initialization that is required.
 *
 * Note:            
 *
 ******************************************************************************/
void UserInit(void)
{
	unsigned char i = 0;
    
	for( i = 0 ; i < 64 ; i++ )
		USB_Out_Buffer[i] = ' ';

    mInitAllLEDs();
    mInitAllSwitches();
    
}//end UserInit

/*******************************************************************************
 * Function:        void ProcessIO(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is a place holder for other user
 *                  routines. It is a mixture of both USB and
 *                  non-USB tasks.
 *
 * Note:            None
 ******************************************************************************/
void ProcessIO(void)
{   
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;
	Lab3a();
	Lab3b();
    CDCTxService();
}//end ProcessIO


/*******************************************************************************
 * Function:        void Lab3a(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        In this function you should write the code to send 
 *                  an array {0x30, 0x31, 0x32, 0x33} to HyperTerminal when
 *                  sw is pressed
 *
 * Note:            None
 ******************************************************************************/
void Lab3a(void)
{
    if (SwitchIsPressed())
    {
		//### Add your code here ###	
    }        
}

/*******************************************************************************
 * Function:        void Lab3b(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        In this function you should write the code to receive 
 *                  an array from HyperTerminal and toggle LED1 if the received
 *                  array is {0x61, 0x62, 0x63, 0x64}
 *
 * Note:            None
 ******************************************************************************/
void Lab3b(void)
{
	//### Add your code here ###	
}

/*******************************************************************************
 * Function:        BOOL SwitchIsPressed(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          Returns TRUE if sw was pressed
 *                  Returns FALSE if sw was not pressed
 *
 * Side Effects:    None
 *
 * Overview:        This function is used to check sw switch.
 *
 * Note:            None
 ******************************************************************************/
BOOL SwitchIsPressed(void)
{
    if(sw != old_sw)
    {
        old_sw = sw;                  // Save new value
        if(sw == 0)                   // If pressed
            return TRUE;              // Was pressed
    }//end if
    return FALSE;                     // Was not pressed
}//end SwitchIsPressed

// ******************************************************************************************************
// ************** USB Callback Functions ****************************************************************
// ******************************************************************************************************

/*******************************************************************
 * Function:        BOOL USER_USB_CALLBACK_EVENT_HANDLER(
 *                        USB_EVENT event, void *pdata, WORD size)
 *
 * PreCondition:    None
 *
 * Input:           USB_EVENT event - the type of event
 *                  void *pdata - pointer to the event data
 *                  WORD size - size of the event data
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called from the USB stack to
 *                  notify a user application that a USB event
 *                  occured.  This callback is in interrupt context
 *                  when the USB_INTERRUPT option is selected.
 *
 * Note:            None
 *******************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch(event)
    {
        case EVENT_CONFIGURED: 
            CDCInitEP();
            break;
        case EVENT_EP0_REQUEST:
            USBCheckCDCRequest();
            break;
        default:
            break;
    }      
    return TRUE; 
}

/** EOF main.c ****************************************************************/
