/********************************************************************
 FileName:     main.c
 Dependencies: See INCLUDES section
 Processor:		PIC18, PIC24 or PIC32 USB Microcontrollers
 Hardware:		The code is natively intended to be used on the following
 				hardware platforms: PICDEM FS USB Demo Board, 
 				PIC18F87J50 FS USB Plug-In Module, or
 				Explorer 16 + PIC24 USB PIM or PIC32 USB PIM.  The firmware may be
 				modified for use on other USB platforms by editing the
 				HardwareProfile.h file.
 Complier:  	Microchip C18 (for PIC18), C30 (for PIC24) or C32 (for PIC32)
 Company:		Microchip Technology, Inc.

 Software License Agreement:

 The software supplied herewith by Microchip Technology Incorporated
 (the Company) for its PIC Microcontroller is intended and
 supplied to you, the Companys customer, for use solely and
 exclusively on Microchip PIC Microcontroller products. The
 software is owned by the Company and/or its supplier, and is
 protected under applicable copyright laws. All rights are reserved.
 Any use in violation of the foregoing restrictions may subject the
 user to criminal sanctions under applicable laws, as well as to
 civil liability for the breach of the terms and conditions of this
 license.

 THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************************
 File Description:

 Change History:
  Rev   Date         Description
  1.0   11/19/2004   Initial release
  2.1   02/26/2007   Updated for simplicity and to use common
                     coding style
  3.0	11/14/2008	 RTC Version
********************************************************************/

/** INCLUDES *******************************************************/
#include "Compiler.h"
#include "HardwareProfile.h"

#if defined(__C32__)
    #include "GenericTypeDefs.h"
    #include <plib.h>
    #define CLOCK_FREQ 60000000
#elif defined(__C30__)
    #define USE_AND_OR
    #include <uart.h>
    #include <PPS.h>
#else
    #define USE_OR_MASKS
    #include "GenericTypeDefs.h"
    #include <usart.h>
#endif

#include "Temperature.h"

/** CONFIGURATION *************************************************************/
#if defined(PICDEM_FS_USB)                  // Configuration bits for PICDEM
                                            // FS USB Demo Board (PIC18F4550)
        #pragma config PLLDIV   = 5         // (20 MHz crystal on PICDEM FS USB
                                            //  board)
        #pragma config CPUDIV   = OSC1_PLL2   
        #pragma config USBDIV   = 2         // Clock source from 96MHz PLL/2
        #pragma config FOSC     = HSPLL_HS
        #pragma config FCMEN    = OFF
        #pragma config IESO     = OFF
        #pragma config PWRT     = OFF
        #pragma config BOR      = ON
        #pragma config BORV     = 3
        #pragma config VREGEN   = ON        // USB Voltage Regulator
        #pragma config WDT      = OFF
        #pragma config WDTPS    = 32768
        #pragma config MCLRE    = ON
        #pragma config LPT1OSC  = OFF
        #pragma config PBADEN   = OFF
        #pragma config STVREN   = ON
        #pragma config LVP      = OFF
        #pragma config XINST    = OFF       // Extended Instruction Set
        #pragma config CP0      = OFF
        #pragma config CP1      = OFF
        #pragma config CPB      = OFF
        #pragma config WRT0     = OFF
        #pragma config WRT1     = OFF
        #pragma config WRTB     = OFF       // Boot Block Write Protection
        #pragma config WRTC     = OFF
        #pragma config EBTR0    = OFF
        #pragma config EBTR1    = OFF
        #pragma config EBTRB    = OFF

#elif defined(LOW_PIN_COUNT_USB_DEVELOPMENT_KIT)
        //14K50
        #pragma config CPUDIV = NOCLKDIV
        #pragma config USBDIV = OFF
        #pragma config FOSC   = HS
        #pragma config PLLEN  = ON
        #pragma config FCMEN  = OFF
        #pragma config IESO   = OFF
        #pragma config PWRTEN = OFF
        #pragma config BOREN  = OFF
        #pragma config BORV   = 30
        #pragma config WDTEN  = OFF
        #pragma config WDTPS  = 32768
        #pragma config MCLRE  = OFF
        #pragma config HFOFST = OFF
        #pragma config STVREN = ON
        #pragma config LVP    = OFF
        #pragma config XINST  = OFF
        #pragma config BBSIZ  = OFF
        #pragma config CP0    = OFF
        #pragma config CP1    = OFF
        #pragma config CPB    = OFF
        #pragma config WRT0   = OFF
        #pragma config WRT1   = OFF
        #pragma config WRTB   = OFF
        #pragma config WRTC   = OFF
        #pragma config EBTR0  = OFF
        #pragma config EBTR1  = OFF
        #pragma config EBTRB  = OFF       

#elif defined(EXPLORER_16)
    #ifdef __PIC24FJ256GB110__              // Defined by MPLAB when using 
                                            // PIC24FJ256GB110 device
        _CONFIG1( JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF & ICS_PGx2) 
        _CONFIG2( 0xF7FF & IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_OFF & POSCMOD_HS & FNOSC_PRIPLL & PLLDIV_DIV2 & IOL1WAY_ON)

    #elif defined(__32MX460F512L__)
        #pragma config UPLLEN   = ON        // USB PLL Enabled
        #pragma config FPLLMUL  = MUL_15    // PLL Multiplier
        #pragma config UPLLIDIV = DIV_2     // USB PLL Input Divider
        #pragma config FPLLIDIV = DIV_2     // PLL Input Divider
        #pragma config FPLLODIV = DIV_1     // PLL Output Divider
        #pragma config FPBDIV   = DIV_1     // Peripheral Clock divisor
        #pragma config FWDTEN   = OFF       // Watchdog Timer
        #pragma config WDTPS    = PS1       // Watchdog Timer Postscale
        #pragma config FCKSM    = CSDCMD    // Clock Switching & Fail Safe Clock
                                            // Monitor
        #pragma config OSCIOFNC = OFF       // CLKO Enable
        #pragma config POSCMOD  = HS        // Primary Oscillator
        #pragma config IESO     = OFF       // Internal/External Switch-over
        #pragma config FSOSCEN  = OFF       // Secondary Oscillator Enable
                                            // (KLO was off)
        #pragma config FNOSC    = PRIPLL    // Oscillator Selection
        #pragma config CP       = OFF       // Code Protect
        #pragma config BWP      = OFF       // Boot Flash Write Protect
        #pragma config PWP      = OFF       // Program Flash Write Protect
        #pragma config ICESEL   = ICS_PGx2  // ICE/ICD Comm Channel Select
        #pragma config DEBUG    = ON        // Background Debugger Enable

    #else
        #error No hardware board defined, see "HardwareProfile.h" and __FILE__
    #endif

#else
    #error No hardware board defined, see "HardwareProfile.h" and __FILE__
#endif

/** P R I V A T E  P R O T O T Y P E S ***************************************/

void UserInit(void);

/** DECLARATIONS ***************************************************/

#define BAUDRATE 57600

/******************************************************************************
 * Function:        void main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 *****************************************************************************/
#pragma code
#if defined(__18CXX)
void main(void)
#else
int main(void)
#endif
{   
    UserInit();

    while(1)
    {
	    while(!SwitchIsPressed());
	    AcquireTemperature();// Perform temperature reading from sensor using "AcquireTemperature()"
        UpdateCelsiusASCII();// Convert to ASCII string using "UpdateCelsiusASCII()
		#if defined(__C30__)
			putsUART2((unsigned int*)tempString);
		#elif defined(__C32__)
			putsUART2(tempString);
		#else
	       	putsUSART(tempString);
		#endif
    }//end while
}//end main

/******************************************************************************
 * Function:        void UserInit(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This routine should take care of all of the demo code
 *                  initialization that is required.
 *
 * Note:            
 *
 *****************************************************************************/
void UserInit(void)
{
   	//Initialize all of the LED pins (see HardwareConfig.h)
   	mInitAllLEDs();
   
   	//Initialize all of the Switch inputs (see HardwareConfig.h)
	mInitAllSwitches();

	//Initialize Temperature Sensor on the Board used in the lab
	InitTempSensor(); 		// see temperature.c
	
	#if defined(__C30__)
	    PPSUnLock;
		IN_FN_PPS_U2RX = IN_PIN_PPS_RP10;
		OUT_PIN_PPS_RP17 = OUT_FN_PPS_U2TX;
		PPSLock;
		TRISFbits.TRISF5=0;
		OpenUART2(UART_EN,
		          UART_TX_ENABLE,
		          ((CLOCK_FREQ/2)/(16*BAUDRATE)-1));
	#elif defined(__C32__)
		TRISFbits.TRISF5=0;
		OpenUART2(UART_EN |
				  UART_NO_PAR_8BIT |
				  UART_1STOPBIT,
				  UART_TX_ENABLE |
				  UART_RX_ENABLE,
				  (CLOCK_FREQ/(16*BAUDRATE)-1) );
	#else
		OpenUSART(USART_BRGH_HIGH,
				  (CLOCK_FREQ/(16*BAUDRATE)-1));
	#endif   
}//end UserInit

